from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
import types
from six import text_type


def format_exception_only(etype, value):
    u'Format the exception part of a traceback.\n\n    The arguments are the exception type and value such as given by\n    sys.last_type and sys.last_value. The return value is a list of\n    strings, each ending in a newline.\n\n    Normally, the list contains a single string; however, for\n    SyntaxError exceptions, it contains several lines that (when\n    printed) display detailed information about where the syntax\n    error occurred.\n\n    The message indicating which exception occurred is always the last\n    string in the list.\n\n    '
    if isinstance(etype, BaseException) or isinstance(etype, types.InstanceType
        ) or etype is None or type(etype) is str:
        return [_format_final_exc_line(etype, value)]
    stype = etype.__name__
    if not issubclass(etype, SyntaxError):
        return [_format_final_exc_line(stype, value)]
    lines = []
    try:
        msg, (filename, lineno, offset, badline) = value.args
    except Exception:
        pass
    else:
        filename = filename or u'<string>'
        lines.append(u'  File "{}", line {}\n'.format(filename, lineno))
        if badline is not None:
            if isinstance(badline, bytes):
                badline = badline.decode(u'utf-8', u'replace')
            lines.append(u'    {}\n'.format(badline.strip()))
            if offset is not None:
                caretspace = badline.rstrip(u'\n')[:offset].lstrip()
                caretspace = (c.isspace() and c or u' ' for c in caretspace)
                lines.append(u'   {}^\n'.format(u''.join(caretspace)))
        value = msg
    lines.append(_format_final_exc_line(stype, value))
    return lines


def _format_final_exc_line(etype, value):
    u'Return a list of a single line -- normal case for format_exception_only'
    valuestr = _some_str(value)
    if value is None or not valuestr:
        line = u'{}\n'.format(etype)
    else:
        line = u'{}: {}\n'.format(etype, valuestr)
    return line


def _some_str(value):
    try:
        return text_type(value)
    except Exception:
        try:
            return bytes(value).decode(u'UTF-8', u'replace')
        except Exception:
            pass
    return u'<unprintable {} object>'.format(type(value).__name__)
