"""Core tests module for wcwidth."""
import wcwidth


def test_hello_jp():
    u'\n    Width of Japanese phrase: \u30b3\u30f3\u30cb\u30c1\u30cf, \u30bb\u30ab\u30a4!\n\n    Given a phrase of 5 and 3 Katakana ideographs, joined with\n    3 English-ASCII punctuation characters, totaling 11, this\n    phrase consumes 19 cells of a terminal emulator.\n    '
    phrase = u'\u30b3\u30f3\u30cb\u30c1\u30cf, \u30bb\u30ab\u30a4!'
    expect_length_each = 2, 2, 2, 2, 2, 1, 1, 2, 2, 2, 1
    expect_length_phrase = sum(expect_length_each)
    length_each = tuple(map(wcwidth.wcwidth, phrase))
    length_phrase = wcwidth.wcswidth(phrase)
    assert length_each == expect_length_each
    assert length_phrase == expect_length_phrase


def test_wcswidth_substr():
    """
    Test wcswidth() optional 2nd parameter, ``n``.

    ``n`` determines at which position of the string
    to stop counting length.
    """
    phrase = u'\u30b3\u30f3\u30cb\u30c1\u30cf, \u30bb\u30ab\u30a4!'
    end = 7
    expect_length_each = 2, 2, 2, 2, 2, 1, 1
    expect_length_phrase = sum(expect_length_each)
    length_phrase = wcwidth.wcswidth(phrase, end)
    assert length_phrase == expect_length_phrase


def test_null_width_0():
    """NULL (0) reports width 0."""
    phrase = u'abc\x00def'
    expect_length_each = 1, 1, 1, 0, 1, 1, 1
    expect_length_phrase = sum(expect_length_each)
    length_each = tuple(map(wcwidth.wcwidth, phrase))
    length_phrase = wcwidth.wcswidth(phrase, len(phrase))
    assert length_each == expect_length_each
    assert length_phrase == expect_length_phrase


def test_control_c0_width_negative_1():
    """CSI (Control sequence initiate) reports width -1."""
    phrase = u'\x1b[0m'
    expect_length_each = -1, 1, 1, 1
    expect_length_phrase = -1
    length_each = tuple(map(wcwidth.wcwidth, phrase))
    length_phrase = wcwidth.wcswidth(phrase, len(phrase))
    assert length_each == expect_length_each
    assert length_phrase == expect_length_phrase


def test_combining_width_negative_1():
    """Simple test combining reports total width of 4."""
    phrase = u'--\u05bf--'
    expect_length_each = 1, 1, 0, 1, 1
    expect_length_phrase = 4
    length_each = tuple(map(wcwidth.wcwidth, phrase))
    length_phrase = wcwidth.wcswidth(phrase, len(phrase))
    assert length_each == expect_length_each
    assert length_phrase == expect_length_phrase


def test_combining_cafe():
    u'Phrase cafe + COMBINING ACUTE ACCENT is caf\xe9 of length 4.'
    phrase = u'cafe\u0301'
    expect_length_each = 1, 1, 1, 1, 0
    expect_length_phrase = 4
    length_each = tuple(map(wcwidth.wcwidth, phrase))
    length_phrase = wcwidth.wcswidth(phrase, len(phrase))
    assert length_each == expect_length_each
    assert length_phrase == expect_length_phrase


def test_combining_enclosing():
    u'CYRILLIC CAPITAL LETTER A + COMBINING CYRILLIC HUNDRED THOUSANDS SIGN is \u0410\u0488 of length 1.'
    phrase = u'\u0410\u0488'
    expect_length_each = 1, 0
    expect_length_phrase = 1
    length_each = tuple(map(wcwidth.wcwidth, phrase))
    length_phrase = wcwidth.wcswidth(phrase, len(phrase))
    assert length_each == expect_length_each
    assert length_phrase == expect_length_phrase


def test_combining_spacing():
    u'Balinese kapal (ship) is \u1b13\u1b28\u1b2e\u1b44 of length 4.'
    phrase = u'\u1b13\u1b28\u1b2e\u1b44'
    expect_length_each = 1, 1, 1, 1
    expect_length_phrase = 4
    length_each = tuple(map(wcwidth.wcwidth, phrase))
    length_phrase = wcwidth.wcswidth(phrase, len(phrase))
    assert length_each == expect_length_each
    assert length_phrase == expect_length_phrase
